/*---------------------------------------------------------------------\
|                                                                      |
|                     _     _   _   _     __     _                     |
|                    | |   | | | \_/ |   /  \   | |                    |
|                    | |   | | | |_| |  / /\ \  | |                    |
|                    | |__ | | | | | | / ____ \ | |__                  |
|                    |____||_| |_| |_|/ /    \ \|____|                 |
|                                                                      |
|                             nfs-server library                       |
|                                                                      |
|                                         (C) SUSE Linux Products GmbH |
|                                         (C) Novell, Inc.             |
\----------------------------------------------------------------------/

  File:       KNfsExports.hpp

  Author:     Marius Tomaschewski, Chakravarthi P
  Maintainer: Chakravarthi P

/-*/
/**
 * @file   KNfsExports.hpp
 * @brief  Provides knfs specific interface for /etc/exports handling.
 *
 * TODO: implement/enable client and options checks,
 *       provide the lists of known and default option
 *       (sets) in the interface.
 */
#ifndef   LIMAL_NFS_SERVER_KNFS_EXPORTS_HPP
#define   LIMAL_NFS_SERVER_KNFS_EXPORTS_HPP

#include <limal/nfs-server/config.h>
#include <limal/nfs-server/NfsExports.hpp>

#include <blocxx/String.hpp>
#include <blocxx/Array.hpp>


// -------------------------------------------------------------------
namespace LIMAL_NAMESPACE
{
namespace NFS_SERVER_NAMESPACE
{


class KNfsExportRules;


// -------------------------------------------------------------------
class KNfsExports: public Exports
{
public:
	KNfsExports();
	KNfsExports(const Exports &exports);
	KNfsExports(const ExportEntries &entries);
	~KNfsExports();

	/**
	 * Set export entries.
	 */
	//void
	//setEntries(const ExportEntries &entries);

	/**
	 * Add export entry.
	 */
	//void
	//addEntry(const ExportEntry    &entry);

	bool
	validExportOptions (const ExportOptions &options) const;

	bool
	validExportClient  (const ExportClient &client) const;

	bool
	validExportEntry   (const ExportEntry &entry) const;

	blocxx::Array<blocxx::String>
	verifyExportOptions(const ExportOptions &options) const;

	blocxx::Array<blocxx::String>
	verifyExportClient (const ExportClient &client) const;

	blocxx::Array<blocxx::String>
	verifyExportEntry  (const ExportEntry &entry) const;


	/**
	 * Loads export entries from the specified exports file
	 * TODO: and checks if all clients and options
	 *       are known to knfs / using correct syntax.
	 *
	 * @param exportsFile The exports file name.
	 */
	void
	load(const blocxx::String &exportsFile);

	/**
	 * Saves current exports to specified exports file name
	 * creating a backup first if backupSuffix is not empty
	 * and the exports file alteady existed.
	 *
	 * @param exportsFile The exports file name.
	 * @param backupSuffix The suffix appended to exportsFile
	 *                     while writing a backup file.
	 */
	void
	save(const blocxx::String &exportsFile,
	     const blocxx::String &backupSuffix);

	/**
	 * Parse specified exports file and return export entries
	 * found in the file.
	 */
	static Exports
	parseExportsFile(const blocxx::String &exportsFile);

	/**
	 * Parse the specified (exports file encoded) line
	 * and construct an export entry object.
	 */
	static ExportEntry
	parseExportEntry(const blocxx::String &exportsLine);

	/**
	 * Saves specified export entries to the exports file name
	 * creating a backup first, if backupSuffix is not empty
	 * and the exports file already existed.
	 * No knfs related checks for export clients and options
	 * are done.
	 *
	 * @param entries      The export entries.
	 * @param exportsFile  The exports file name.
	 * @param backupSuffix The suffix appended to exportsFile
	 *                     while writing a backup file.
	 */
	static void
	writeExportsFile(const Exports        &exports,
	                 const blocxx::String &exportsFile,
	                 const blocxx::String &backupSuffix);

	/**
	 * Decode first chunk of the string.
	 */
	static blocxx::String
	decodeSubStr(blocxx::String &str, const blocxx::String &stopAt);

	/**
	 * Encode string to a form that can be written into a exports file.
	 * If exceptSpaces is true, the resulting string may be surrounded
	 * with quote marks ("") if it contains spaces.
	 * If exceptSpaces is false, all spaces are escaped to octal nmbers.
	 */
	static blocxx::String
	encodeString(const blocxx::String &str, bool exceptSpaces=false);

private:
	KNfsExportRules *rules;
	bool             dirty;
};


// -------------------------------------------------------------------
}      // End of NFS_SERVER_NAMESPACE
}      // End of LIMAL_NAMESPACE

#endif // LIMAL_NFS_SERVER_KNFS_EXPORTS_HPP
// vim: set ts=8 sts=8 sw=8 ai noet:
