/*---------------------------------------------------------------------\
|                                                                      |
|                     _     _   _   _     __     _                     |
|                    | |   | | | \_/ |   /  \   | |                    |
|                    | |   | | | |_| |  / /\ \  | |                    |
|                    | |__ | | | | | | / ____ \ | |__                  |
|                    |____||_| |_| |_|/ /    \ \|____|                 |
|                                                                      |
|                             nfs-server library                       |
|                                                                      |
|                                         (C) SUSE Linux Products GmbH |
|                                         (C) Novell, Inc.             |
\----------------------------------------------------------------------/

  File:       KNfsIdmapdConf.hpp

  Author:     Marius Tomaschewski, Chakravarthi P
  Maintainer: Chakravarthi P

/-*/
/**
 * @file   KNfsIdmapdConf.hpp
 * @brief  Provides interface for /etc/idmapd.conf handling.
 */
#ifndef   LIMAL_NFS_SERVER_KNFS_IDMAPD_CONF_HPP
#define   LIMAL_NFS_SERVER_KNFS_IDMAPD_CONF_HPP

#include <limal/nfs-server/config.h>
#include <limal/nfs-server/NfsIdmapdConf.hpp>

#include <blocxx/String.hpp>
#include <blocxx/Array.hpp>


// -------------------------------------------------------------------
namespace LIMAL_NAMESPACE
{
namespace NFS_SERVER_NAMESPACE
{


class KNfsIdmapdRules;


// -------------------------------------------------------------------
class KNfsIdmapdConf: public IdmapdConf
{
public:
	KNfsIdmapdConf();
	KNfsIdmapdConf(const IdmapdConf &conf);
	~KNfsIdmapdConf();

	bool
	validIdmapdEntry (const blocxx::String &name,
		      const blocxx::String &value) const;

	blocxx::String
	verifyIdmapdEntry (const IdmapdEntry &entry) const;

	blocxx::Array<blocxx::String>
	verifyIdmapdSection (const IdmapdSection &section) const;

	/**
	 * checks for essential sections and dependencies
	 * apart from checking individual sections using
	 * verifyIdmapdSection () 
	 */
	const blocxx::Array<blocxx::String>
	verifyConf ();

	/**
	 * Loads idmapd entries from the specified idmapd file
	 * @param idmapdFile The idmapd file name.
	 */
	void
	load(const blocxx::String &filename);

	/**
	 * Saves current idmapd entries to specified idmapd file name
	 * creating a backup first if backupSuffix is not empty
	 * and the idmapd file alteady existed.
	 *
	 * @param idmapdFile The idmapd file name.
	 * @param backupSuffix The suffix appended to idmapdFile
	 *                     while writing a backup file.
	 */
	void
	save(const blocxx::String &idmapdFile,
	     const blocxx::String &backupSuffix);

	/**
	 * Parse specified idmapd file and return a IdmapConf object
	 * built there of .
	 */
	static IdmapdConf
	parseIdmapdConfFile(const blocxx::String &idmapdFile);

	/**
	 * Parse the specified (idmapd file encoded) line
	 * and construct an Idmapd entry object.
	 */
	static IdmapdEntry
	parseIdmapdEntry(const blocxx::String &entry);

	/**
	 * Saves specified idmapd entries to the idmapd file name
	 * creating a backup first, if backupSuffix is not empty
	 * and the idmapd file already existed.
	 *
	 * @param entries      The idmapd entries.
	 * @param idmapdFile  The idmapd file name.
	 * @param backupSuffix The suffix appended to idmapdFile
	 *                     while writing a backup file.
	 */
	static void
	writeIdmapdConfFile(const IdmapdConf        &idmapdConf,
	                 const blocxx::String &idmapdFile,
	                 const blocxx::String &backupSuffix);

private:
	KNfsIdmapdRules *rules;
	bool             dirty;
};


// -------------------------------------------------------------------
}      // End of NFS_SERVER_NAMESPACE
}      // End of LIMAL_NAMESPACE

#endif // LIMAL_NFS_SERVER_KNFS_IDMAPD_CONF_HPP
// vim: set ts=8 sts=8 sw=8 ai 
