/*---------------------------------------------------------------------\
|                                                                      |
|                     _     _   _   _     __     _                     |
|                    | |   | | | \_/ |   /  \   | |                    |
|                    | |   | | | |_| |  / /\ \  | |                    |
|                    | |__ | | | | | | / ____ \ | |__                  |
|                    |____||_| |_| |_|/ /    \ \|____|                 |
|                                                                      |
|                             nfs-server library                       |
|                                                                      |
|                                         (C) SUSE Linux Products GmbH |
|                                         (C) Novell, Inc.             |
\----------------------------------------------------------------------/

  File:       NfsExports.hpp

  Author:     Marius Tomaschewski, Chakravarthi P
  Maintainer: Chakravarthi P

/-*/
/**
 * @file   NfsExports.hpp
 * @brief  Provides interface for /etc/exports handling.
 */
#ifndef   LIMAL_NFS_SERVER_NFS_EXPORTS_HPP
#define   LIMAL_NFS_SERVER_NFS_EXPORTS_HPP

#include <limal/nfs-server/config.h>
#include <blocxx/String.hpp>
#include <blocxx/Array.hpp>
#include <blocxx/List.hpp>
#include <blocxx/Map.hpp>


// -------------------------------------------------------------------
namespace LIMAL_NAMESPACE
{
namespace NFS_SERVER_NAMESPACE
{


// -------------------------------------------------------------------
/**
 * @class ExportOption
 * @brief The ExportOption class manages a nfs export option.
 *
 * A ExportOption consists of either just a "name" (flag option)
 * or a "name=value" pair.
 */
class ExportOption
{
public:
	/**
	 * Create a new, empty export option
	 */
	ExportOption();

	/**
	 * Create a new export option object using the
	 * specified name and (may be empty) value.
	 *
	 * @code
	 *	ExportOption("ro", "");
	 *	ExportOption("mountpoint", "/some/path");
	 * @endcode
	 *
	 * @param name  The export option name.
	 * @param value The export option value.
	 * @throws limal::ValueException if the string
	 *         is not a valid export option.
	 */
	ExportOption(const blocxx::String &name,
	             const blocxx::String &value);

	/**
	 * Create a new export option object using the
	 * specified string containing either only the
	 * option "name" or a "name=value" pair.
	 *
	 * @code
	 *	ExportOption("ro");
	 *	ExportOption("mountpoint=/some/path");
	 * @endcode
	 *
	 * @param option The export option as string.
	 * @throws limal::ValueException if the string
	 *         is not a valid export option.
	 */
	ExportOption(const blocxx::String &option);

	/**
	 * Create export option as copy of the specified one.
	 *
	 * @param option A export option reference.
	 */
	ExportOption(const ExportOption &option);

	/**
	 * Destroy current export option object.
	 */
	~ExportOption();

	/**
	 * Returns true if the option is empty.
	 *
	 * @return true if the option is empty
	 */
	bool                    empty() const;

	/**
	 * Return the export option as string. This is either
	 * only the "name" or "name=value" if value not empty.
	 *
	 * @return The export option as string.
	 */
	blocxx::String          toString()  const;

	/**
	 * Get the name of the export option
	 *
	 * @return export option name
	 */
	blocxx::String          getName()  const;

	/**
	 * Get the value of the export option
	 *
	 * @return export option value
	 */
	blocxx::String          getValue() const;

	/**
	 * Set the name of the option and reset the value
	 * to a empty string.
	 */
	void                    setName(const blocxx::String &name);

	/**
	 * Set the value of the option.
	 */
	void                    setValue(const blocxx::String &value);

	/**
	 * Assign the export option object to the specified string
	 * containing either just the option "name" or "name=value".
	 *
	 * @code
	 *	ExportOption e;
	 *      e.setOption("ro");
	 *      e.setOption("mountpoint=/some/path");
	 * @endcode
	 *
	 * @param option The export option as string.
	 * @throws limal::ValueException if the string
	 *         is not a valid export option.
	 */
	void                    setOption(const blocxx::String &option);

#ifndef SWIG
	friend bool
	operator == (const ExportOption & o1, const ExportOption & o2);

	friend bool
	operator <  (const ExportOption & o1, const ExportOption & o2);
#endif

private:
	blocxx::String  m_name;
	blocxx::String  m_value;
};


// -------------------------------------------------------------------
/**
 * @class ExportOptions
 * @brief ExportOption list.
 */
typedef blocxx::List<ExportOption>      ExportOptions;


// -------------------------------------------------------------------
/**
 * @class ExportClient
 * @brief Export client specification inclusive its option list.
 *
 * A ExportClient consists of its specification (e.g. hostname)
 * followed by a comma-separated export option list enlosed in (),
 * e.g. "dhcp*.local.domain(ro,sync,root_squash)".
 */
class ExportClient
{
public:
	/**
	 * Construct a new, empty export client object
	 */
	ExportClient();

	/**
	 * Construct a new export client as copy of the specified one.
	 *
	 * @param client The export client to copy.
	 */
	ExportClient(const ExportClient   &client);

	/**
	 * Construct a new export client using the specified name
	 * and list of export options.
	 *
	 * @code
	 *	ExportOptions opts;
	 *      opts.push_back(ExportOption("ro"));
	 *      opts.push_back(ExportOption("mountpoint=/some/path"));
	 *
	 *	ExportClient("127.0.0.1", opts);
	 * @endcode
	 *
	 * @param client  The export client name.
	 * @param options The export option list.
	 */
	ExportClient(const blocxx::String &client,
	             const ExportOptions  &options);

	/**
	 * Construct a new export client using the specified name
	 * and string with comma-separated export options.
	 *
	 * @code
	 *	ExportClient("127.0.0.1", "ro,sync,mountpoint=/some/path");
	 * @endcode
	 *
	 * @param client The export client name.
	 * @param options The string with export options.
	 */
	ExportClient(const blocxx::String &client,
	             const blocxx::String &options);

	/**
	 * Construct a new export client using the specified string
	 * representation of a export client, that is containing the
	 * client name followed by a string of comma-separated export
	 * options enclosed in ().
	 *
	 * @code
	 *	ExportClient("127.0.0.1(ro,sync,mountpoint=/some/path)");
	 * @endcode
	 *
	 * @param clientAndOpts The string representation the export client.
	 */
	ExportClient(const blocxx::String &clientAndOpts);

	/**
	 * Destroy the export client object.
	 */
	~ExportClient();

	/**
	 * Returns true if the export client object doesn't contain
	 * any name and options.
	 *
	 * @return true if the export client is empty.
	 */
	bool                    empty() const;

	/**
	 * Returns the client name, followed by a comma-separated list
	 * of export options enclosed in ().
	 *
	 * @return The string representation the export client.
	 */
	blocxx::String          toString() const;

	/**
	 * Returns the export client name.
	 *
	 * @return The export client name.
	 */
	blocxx::String          getClient() const;

	/**
	 * Returns the export options list.
	 *
	 * @return The export options list.
	 */
	ExportOptions           getOptions() const;

	/**
	 * Returns the comma-separated export options.
	 *
	 * @return The export options string.
	 */
	blocxx::String          getOptionsString() const;

	/**
	 * Sets the export client name to the specified string.
	 *
	 * @param client The client name string.
	 */
	void                    setClient (const blocxx::String &client);

	/**
	 * Sets the export option list.
	 *
	 * @param options The export option list.
	 */
	void                    setOptions(const ExportOptions  &options);

	/**
	 * Sets the export option list.
	 *
	 * @param options The export option list as string.
	 */
	void                    setOptions(const blocxx::String  &options);

	/**
	 * Parses the specified string as a list of comma-separated export
	 * options.
	 */
	static
	ExportOptions           parseOptions(const blocxx::String &options);

#ifndef SWIG
	friend bool
	operator == (const ExportClient &c1, const ExportClient &c2);

	friend bool
	operator <  (const ExportClient &c1, const ExportClient &c2);
#endif

private:
	blocxx::String  m_client;
	ExportOptions   m_options;
};


// -------------------------------------------------------------------
/**
 * @class ExportClients
 * @brief ExportClient list.
 */
typedef blocxx::List<ExportClient>      ExportClients;


// -------------------------------------------------------------------
/**
 * @class Comment
 * @brief List of ExportEntry and Exports file comment lines.
 */
typedef blocxx::Array<blocxx::String>	ExportComment;


// -------------------------------------------------------------------
/**
 * @class ExportEntry
 * @brief Export entry path with a list of client specifications.
 */
class ExportEntry
{
public:
	/**
	 * Construct a new, empty export entry.
	 */
	ExportEntry();

	/**
	 * Construct a new export entry as copy of the specified one.
	 *
	 * @param client The export client to copy.
	 */
	ExportEntry(const ExportEntry &entry);


	/**
	 * Construct a new export entry using the specified path
	 * and client list.
	 *
	 * @param path The export path.
	 * @param clients The list of export clients.
	 */
	ExportEntry(const blocxx::String &path,
	            const ExportClients  &clients);

	/**
	 * Construct a new export entry using the specified path,
	 * client list and comment lines.
	 *
	 * @param path The export path.
	 * @param clients The list of export clients.
	 * @param comment The list comment lines for this entry.
	 */
	ExportEntry(const blocxx::String &path,
	            const ExportClients  &clients,
	            const ExportComment  &comment);

	/**
	 * Destroy the export entry object.
	 */
	~ExportEntry();

	/**
	 * Returns true if no path and clients are set.
	 *
	 * @return true if no path and clients are set.
	 */
	bool                    empty() const;

	/**
	 * @return The export path and a space separated list of clients.
	 */
	blocxx::String          toString() const;

	/**
	 * @return The a string array containing the export path followed
	 *         by the list of export clients in string format.
	 */
	blocxx::StringArray	toStringArray() const;

	/**
	 * @return The export path.
	 */
	blocxx::String          getPath() const;

	/**
	 * @return The list of export clients.
	 */
	ExportClients           getClients() const;

	/**
	 * @return The comment lines of this entry.
	 */
	ExportComment           getComment() const;

	/**
	 * Sets the entry's export path to the specified one.
	 *
	 * @param clients The new export path.
	 */
	void                    setPath(const blocxx::String &path);

	/**
	 * Sets the entry's client list to the specified one.
	 *
	 * @param clients The new export clients list.
	 */
	void                    setClients(const ExportClients  &clients);

	/**
	 * Sets the comment for this entry.
	 *
	 * @param comment The new entry comment.
	 */
	void                    setComment(const ExportComment &comment);

	/**
	 * Adds the specified export client to the entry, but only if
	 * the entry does not contain any client with the same name.
	 *
	 * @param client The export client to add.
	 * @return true, if the client was added.
	 */
	bool                    addClient(const ExportClient &client);

	/**
	 * Removes the first export client with the specified client name
	 * from the enrty.
	 *
	 * @param cname The export client name to search for.
	 * @return true, if a client was removed.
	 */
	bool                    removeClient(const blocxx::String &cname);

	/**
	 * Removes the first export client that is equal to the specified one.
	 *
	 * @param client The export client to search for.
	 * @return true, if a client was removed.
	 */
	bool                    removeClient(const ExportClient   &client);

	/**
	 * Removes all the clients.
	 * just clear the m_clients
	*/
	void			removeClients();

#ifndef SWIG
	friend bool
	operator == (const ExportEntry &e1, const ExportEntry &e2);

	friend bool
	operator <  (const ExportEntry &e1, const ExportEntry &e2);
#endif

private:
	blocxx::String  m_path;
	ExportClients   m_clients;
	ExportComment   m_comment;
};


// -------------------------------------------------------------------
/**
 * @class ExportEntries
 * @brief ExportEntry list.
 */
typedef blocxx::List<ExportEntry>       ExportEntries;
typedef blocxx::List<blocxx::String>	ExportPaths;


// -------------------------------------------------------------------
/**
 * @class ExportEntry
 * @brief Exports file in-memory representation.
 */
class Exports
{
public:
	enum CommentType
	{
		E_LEADING,
		E_TRAILING
	};

	/**
	 * Construct a new, empty exports object.
	 */
	Exports();

	/**
	 * Construct a new exports object as a copy of the specified one.
	 *
	 * @param exports The exports object to copy.
	 */
	Exports(const Exports &exports);

	/**
	 * Construct a new set exports object as a copy of the specified
	 * list of export entries.
	 *
	 * @param entries The list of export entries to copy.
	 */
	Exports(const ExportEntries &entries);

	/**
	 * Destroy the exports object.
	 */
	~Exports();

	/**
	 * @return A list of uniqe paths.
	 */
	ExportPaths		getPaths() const;

	/**
	 * @return All clients from entries with the specified path.
	 */
	ExportClients           getClients(const blocxx::String &path) const;

	/**
	 * @return All entries with the specified path.
	 */
	ExportEntries           getEntries(const blocxx::String &path) const;

	/**
	 * @return All entries in this Exports object.
	 */
	ExportEntries           getEntries() const;

	/**
	 * @return The leading or trailing comment lines.
	 */
	ExportComment           getComment(CommentType type) const;

	/**
	 * Replace the current list of entries with the specified one.
	 *
	 * @param entries The new list of export entries.
	 */
	void                    setEntries(const ExportEntries &entries);

	/**
	 * Sets the leading or trailing comment.
	 *
	 * @param comment The new comment.
	 * @param comment The type of the comment.
	 */
	void                    setComment(const ExportComment &comment,
	                                   CommentType         type);

	/**
	 * Add the specified entry to the list.
	 * making sure to resolve duplicates in m_entries.
	 * If the export path is already is used in one of the
	 * existing entries we need to add the clients of 'entry'
	 * uniquely to that existing entry.
	 *
	 * @param entry The entry to add.
	 */
	void                    addEntry(const ExportEntry     &entry);


private:
	ExportEntries m_entries;
	ExportComment l_comment;
	ExportComment t_comment;

};


// -------------------------------------------------------------------
}      // End of NFS_SERVER_NAMESPACE
}      // End of LIMAL_NAMESPACE

#endif // LIMAL_NFS_SERVER_NFS_EXPORTS_HPP
// vim: set ts=8 sts=8 sw=8 ai noet:
