/*---------------------------------------------------------------------\
|                                                                      |
|                     _     _   _   _     __     _                     |
|                    | |   | | | \_/ |   /  \   | |                    |
|                    | |   | | | |_| |  / /\ \  | |                    |
|                    | |__ | | | | | | / ____ \ | |__                  |
|                    |____||_| |_| |_|/ /    \ \|____|                 |
|                                                                      |
|                             nfs-server library                       |
|                                                                      |
|                                         (C) SUSE Linux Products GmbH |
|                                         (C) Novell, Inc.             |
\----------------------------------------------------------------------/

  File:      NfsIdmapdConf.hpp

  Author:     Marius Tomaschewski, Chakravarthi P
  Maintainer: Chakravarthi P

/-*/
/**
 * @file   NfsIdmapdConf.hpp
 * @brief  Interface for /etc/idmapd.conf 
 */
#ifndef   LIMAL_NFS_SERVER_NFS_IDMAPD_CONF_HPP
#define   LIMAL_NFS_SERVER_NFS_IDMAPD_CONF_HPP

#include <limal/nfs-server/config.h>
#include <blocxx/String.hpp>
#include <blocxx/Array.hpp>
#include <blocxx/List.hpp>
#include <blocxx/Map.hpp>

// -------------------------------------------------------------------
namespace LIMAL_NAMESPACE
{
namespace NFS_SERVER_NAMESPACE
{

// -------------------------------------------------------------------
/**
 * @class IdmapdEntry
 * @brief IdmapdEntry is for capturing name=value entry in idmapd.conf
 *
 * An IdmapdEntry is a name=value pair that is used by the idmapd daemon
 * Each IdmapdEntry falls under a particular section (the class IdmapdSection)
 * of the configuration file idmapd.conf
 */
class IdmapdEntry
{
public:
	/**
	 * Create a new, empty Idmapd Entry
	 */
	IdmapdEntry ();

	/**
	 * Create a new idmapd entry object using the specified name
	 * and value . Neither name, nor the value can be empty.
	 *
	 * @code
	 *	IdmapdEntry ("Verbosity,"0");
	 *	IdmapdEntry ("Nobody-User","nobody");
	 * @endcode
	 *
	 * @param name	The idmapd entry name
	 * @param value The idmapd entry value
	 * @throws limal::ValueException if the name is not a valid
	 * entry name and/or the value is not a valid value.
	 */
	IdmapdEntry (const blocxx::String &name,
		     const blocxx::String &value);

	/**
	 * Create a new idmapd entry object using the whole entry line
	 * as it would appear in the idmapd.conf file.
	 *
	 * @code
	 *	IdmapdEntry ("Verbosity = 0");
	 *	IdmapdEntry ("Nobody-User = nobody");
	 * @endcode
	 *
	 * @param name	The idmapd entry name
	 * @param value The idmapd entry value
	 * @throws limal::ValueException if the name is not a valid
	 * entry name and/or the value is not a valid value.
	 */
	IdmapdEntry (const blocxx::String &entry);

	IdmapdEntry (const IdmapdEntry &ientry);

	~IdmapdEntry ();

	bool
	empty () const;

	blocxx::String
	toString () const;
	
	blocxx::String
	getName () const;

	blocxx::String
	getValue () const;

	blocxx::String
	getEntry () const;

	void
	setEntry (const blocxx::String &entry);

	void
	addComment (const blocxx::String &comment);

	bool
	hasComments () const;

	blocxx::Array<blocxx::String>
	getComments () const;



private:
	blocxx::String	m_name;
	blocxx::String	m_value;

	blocxx::Array<blocxx::String> comments;

	void
	setName (const blocxx::String &name);

};

// -------------------------------------------------------------------
/**
 * @class IdmapdEntries
 * @brief IdmapdEntry list.
 */
typedef blocxx::List<IdmapdEntry> IdmapdEntries;


// -------------------------------------------------------------------
class IdmapdSection
{

public:
	typedef blocxx::IntrusiveReference<IdmapdEntry> IERef;
	
	IdmapdSection ();
	
	IdmapdSection (const blocxx::String &name);

	IdmapdSection (const IdmapdSection &section);

	IdmapdSection  (const blocxx::String &name,
			const IdmapdEntries &entries);

	// see if its already there ...
	IdmapdSection &
	addEntry (const IdmapdEntry &entry);

	blocxx::String
	getName () const;

	bool
	empty() const;

	blocxx::Array<blocxx::String>
	getStringEntries () const;

	IdmapdEntry&
	getEntry (const blocxx::String &name) ;

	void	
	setEntry (const IdmapdEntry &entry);
	
	void	
	setEntryValue (const blocxx::String &entry,
		  const blocxx::String &value);

	void
	addComment (const blocxx::String &comment);

	bool
	hasComments () const;

	bool
	hasEntry (const blocxx::String &entryName) const;

	blocxx::Array<blocxx::String>
	getComments () const;


	blocxx::List<IdmapdEntry>
	getEntries () const;

private:

	blocxx::String	m_name;

	/*  IdmapdEntry list */
	IdmapdEntries m_entries;
	blocxx::Map<blocxx::String,IdmapdEntry> entryMap; 

	void	
	setEntryMap (const blocxx::List<IdmapdEntry> &entries);

	blocxx::Array<blocxx::String> comments;
};




// -------------------------------------------------------------------
/**
 * @class IdmapdSections
 * @brief IdmapdSection list.
 */
typedef blocxx::List<IdmapdSection> IdmapdSections;



// -------------------------------------------------------------------
class IdmapdConf
{
public:
	IdmapdConf();

	IdmapdConf(const IdmapdConf &conf);
	
	~IdmapdConf ();

	blocxx::List<IdmapdEntry>
	getEntries (const blocxx::String &sectionName);

	blocxx::Array<blocxx::String>
	getStringEntries (const blocxx::String &sectionName);

	blocxx::Array<blocxx::String>
	getAllStringEntries () const;

	const IdmapdSection&
	getSection (const blocxx::String &sectionName);

	const blocxx::String
	getValueFor (const blocxx::String &entryName);

	bool
	hasSection (const blocxx::String &secName) const;

	blocxx::List<IdmapdSection>
	getSections () const;

	blocxx::Map<blocxx::String,IdmapdSection>
	getSectionMap () const;

	void
	setSectionMap ();

	void
	setSection (const IdmapdSection &section);

	void
	setEntry (const blocxx::String &entryName,
		  const blocxx::String &value);

	void
	addComment (const blocxx::String &comment);

	bool
	hasComments () const;

	blocxx::Array<blocxx::String>
	getComments () const;

	bool
	empty () const;

	/*
	bool
	setIdmapdSection (const blocxx::String &sectionName,
			  const IdmapdSection &section);
	*/	
	
private:
	blocxx::Map<blocxx::String,IdmapdSection> sectionMap;
	blocxx::Array<blocxx::String> comments;
	
};

// -------------------------------------------------------------------



}      // End of NFS_SERVER_NAMESPACE
}      // End of LIMAL_NAMESPACE

#endif // LIMAL_NFS_SERVER_NFS_IDMAPD_CONF_HPP
// vim: set ts=8 sts=8 sw=8 ai noet:
